import { Awaitable } from '@antfu/utils';
import { SnapshotStateType, SnapshotResolver } from 'jest-snapshot';
import { SnapshotStateOptions } from 'jest-snapshot/build/State';

interface UncheckedSnapshot {
    filePath: string;
    keys: Array<string>;
}
interface SnapshotSummary {
    added: number;
    didUpdate: boolean;
    failure: boolean;
    filesAdded: number;
    filesRemoved: number;
    filesRemovedList: Array<string>;
    filesUnmatched: number;
    filesUpdated: number;
    matched: number;
    total: number;
    unchecked: number;
    uncheckedKeysByFile: Array<UncheckedSnapshot>;
    unmatched: number;
    updated: number;
}

interface Context {
    file: string;
    title?: string;
    fullTitle?: string;
}
declare class SnapshotManager {
    snapshotState: SnapshotStateType | null;
    snapshotSummary: SnapshotSummary;
    snapshotOptions: SnapshotStateOptions;
    ctx: Context | null;
    testFile: string;
    snapshotResolver: SnapshotResolver | null;
    rootDir: string;
    constructor({ rootDir, update, snapshotResolver, }: {
        rootDir: string;
        update?: boolean;
        snapshotResolver?: SnapshotResolver | null;
    });
    onFileChanged(): void;
    setTest(test: Test): void;
    setContext(context: Context): void;
    assert(received: unknown, message: string): void;
    clear(): void;
    saveSnap(): void;
    report(): string[] | undefined;
}

interface UserOptions {
    /**
     * Include globs for test files
     *
     * @default ['**\/*.test.ts']
     */
    includes?: string[];
    /**
     * Exclude globs for test files
     * @default ['**\/node_modules\/**']
     */
    excludes?: string[];
    /**
     * Handling for dependencies inlining or externalizing
     */
    deps?: {
        external?: (string | RegExp)[];
        inline?: (string | RegExp)[];
    };
    /**
     * Register apis globally
     *
     * @default false
     */
    global?: boolean;
    /**
     * Use `jsdom` or `happy-dom` to mock browser APIs
     *
     * @default false
     */
    dom?: boolean | 'jsdom' | 'happy-dom';
    /**
     * Update snapshot files
     *
     * @default false
     */
    update?: boolean;
    /**
     * Watch mode
     *
     * @default false
     */
    watch?: boolean;
    /**
     * Project root
     */
    root?: string;
    /**
     * Custom reporter for output
     */
    reporter?: Reporter;
    filters?: string[];
    config?: string | undefined;
}
interface ResolvedConfig extends Omit<Required<UserOptions>, 'config' | 'filters'> {
    config?: string;
    filters?: string[];
    depsInline: (string | RegExp)[];
    depsExternal: (string | RegExp)[];
}
declare type RunMode = 'run' | 'skip' | 'only' | 'todo';
declare type TestState = RunMode | 'pass' | 'fail';
declare type ComputeMode = 'serial' | 'concurrent';
interface TaskBase {
    name: string;
    mode: RunMode;
    computeMode: ComputeMode;
    suite?: Suite;
    file?: File;
    result?: TaskResult;
}
interface TaskResult {
    state: TestState;
    start: number;
    end?: number;
    error?: unknown;
}
interface Suite extends TaskBase {
    type: 'suite';
    tasks: Task[];
}
interface File extends Suite {
    filepath: string;
}
interface Test extends TaskBase {
    type: 'test';
    suite: Suite;
    result?: TaskResult;
}
declare type Task = Test | Suite | File;
declare type TestFunction = () => Awaitable<void>;
interface ConcurrentCollector {
    (name: string, fn: TestFunction): void;
    only: (name: string, fn: TestFunction) => void;
    skip: (name: string, fn: TestFunction) => void;
    todo: (name: string) => void;
}
interface OnlyCollector {
    (name: string, fn: TestFunction): void;
    concurrent: (name: string, fn: TestFunction) => void;
}
interface SkipCollector {
    (name: string, fn: TestFunction): void;
    concurrent: (name: string, fn: TestFunction) => void;
}
interface TodoCollector {
    (name: string): void;
    concurrent: (name: string) => void;
}
interface TestCollector {
    (name: string, fn: TestFunction): void;
    concurrent: ConcurrentCollector;
    only: OnlyCollector;
    skip: SkipCollector;
    todo: TodoCollector;
}
declare type HookListener<T extends any[]> = (...args: T) => Awaitable<void>;
interface SuiteHooks {
    beforeAll: HookListener<[Suite]>[];
    afterAll: HookListener<[Suite]>[];
    beforeEach: HookListener<[Test, Suite]>[];
    afterEach: HookListener<[Test, Suite]>[];
}
interface SuiteCollector {
    readonly name: string;
    readonly mode: RunMode;
    type: 'collector';
    test: TestCollector;
    tasks: (Suite | Test | SuiteCollector)[];
    collect: (file?: File) => Promise<Suite>;
    clear: () => void;
    on: <T extends keyof SuiteHooks>(name: T, ...fn: SuiteHooks[T]) => void;
}
declare type TestFactory = (test: (name: string, fn: TestFunction) => void) => Awaitable<void>;
interface RunnerContext {
    filesMap: Record<string, File>;
    files: File[];
    tests: Test[];
    config: ResolvedConfig;
    reporter: Reporter;
    snapshotManager: SnapshotManager;
}
interface GlobalContext {
    tasks: (SuiteCollector | Test)[];
    currentSuite: SuiteCollector | null;
}
interface Reporter {
    onStart?: (config: ResolvedConfig) => Awaitable<void>;
    onCollected?: (files: File[], ctx: RunnerContext) => Awaitable<void>;
    onFinished?: (ctx: RunnerContext, files?: File[]) => Awaitable<void>;
    onTestBegin?: (test: Test, ctx: RunnerContext) => Awaitable<void>;
    onTestEnd?: (test: Test, ctx: RunnerContext) => Awaitable<void>;
    onSuiteBegin?: (suite: Suite, ctx: RunnerContext) => Awaitable<void>;
    onSuiteEnd?: (suite: Suite, ctx: RunnerContext) => Awaitable<void>;
    onWatcherStart?: (ctx: RunnerContext) => Awaitable<void>;
    onWatcherRerun?: (files: string[], trigger: string, ctx: RunnerContext) => Awaitable<void>;
}

export { ComputeMode as C, File as F, GlobalContext as G, HookListener as H, ResolvedConfig as R, SuiteCollector as S, TestFactory as T, UserOptions as U, TestFunction as a, SuiteHooks as b, RunMode as c, TestState as d, TaskBase as e, TaskResult as f, Suite as g, Test as h, Task as i, TestCollector as j, RunnerContext as k, Reporter as l };
